/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "attr.h"
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "pmhash.h"

int attr_set(struct pmhash **pmhash, char *key, char *val)
{
	int r = -ENOMEM;
	key = strdup(key);
	if (!key)
		return r;
	val = strdup(val);
	if (!val) {
		free(val);
		return r;
	}
	r = pmhash_add(pmhash, key, val);
	if (r) {
		free(key);
		free(val);
	}
	return r;
}

int attr_parse(struct pmhash **pmhash, char *arg)
{
	int r = -EINVAL;
	char *val;
	val = strchr(arg, '=');
	if (!val)
		return r;
	*val++ = '\0';
	return attr_set(pmhash, arg, val);
}

char * attr_get(struct pmhash *pmhash, char *key)
{
	return (char *) pmhash_get(pmhash, key);
}

int attr_out(FILE *f, struct pmhash *attr, char *key, int size)
{
	return fprintf(f, "%-*.*s", size, size, attr_get(attr, key) ?: "");
}

int attr_out_int(FILE *f, struct pmhash *attr, char *key, int size)
{

	/*
	 * As there is no validation of the string when parsed, there is no
	 * point returning any error when there is a parsing failure here. We
	 * emit only a warning, then. Unfortunately, there is no line
	 * information here.
	 */

	char *val;
	unsigned long long num = 0;
	val = attr_get(attr, key);
	if (val) {
		char *end;
		errno = 0;
		num = strtoull(val, &end, 0);
		if ((end && *end != '\0') || errno) {
			fprintf(stderr, "Found invalid value in attribute %s: %s: not a number\n", key, val);
			num = 0;
		}
	}

	return fprintf(f, "%0*llu", size, num);
}
