/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "bem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "cmd.h"
#include "list.h"
#include "util.h"
#include "totais.h"
#include "attr.h"

void bem_free(void *pointer)
{
	struct bem *bem = pointer;
	if (bem->descricao)
		free(bem->descricao);
	if (bem->attr)
		pmhash_del(bem->attr);
	free(bem);
}

static int bem_cmp(void *p1, void *p2)
{
	struct bem *b1 = p1;
	struct bem *b2 = p2;
	/* O bem de valor maior vem primeiro. */
	if (b1->valor > b2->valor)
		return -1;
	else if (b1->valor < b2->valor)
		return 1;
	return 0;
}

static struct bem * bem_new(char **args, int argc)
{
	struct bem *bem;
	int r = 0;
	int i;
	bem = malloc(sizeof(*bem));
	memset(bem, 0, sizeof(*bem));
	bem->attr = pmhash_new();
	if (!bem->attr) {
		bem_free(bem);
		return NULL;
	}

	r += set_int(args[1], &bem->codigo);
	r += set_string(args[2], &bem->descricao);
	r += set_llong(args[3], &bem->valor_anterior);
	r += set_llong(args[4], &bem->valor);

	for (i = 5; i < argc; i++) {
		printf("parsing arg %s\n", args[i]);
		r = attr_parse(&bem->attr, args[i]);
		if (r) {
			bem_free(bem);
			return NULL;
		}
	}

	bem->dependente = 0; /* TODO */

	if (r < 0 || bem->codigo < 0 ||
	    bem->valor_anterior < 0 || bem->valor < 0) {
		bem_free(bem);
		return NULL;
	}
	return bem;
}

static int run_bem(struct declaracao *dec, char **args, int argc)
{
	struct bem *bem;
	int r;
	if (argc < 5)
		return -EINVAL;
	bem = bem_new(args, argc);
	if (!bem)
		return -ENOMEM;
	r = list_insert_ordered(&dec->bens, bem, bem_cmp);
	if (r < 0) {
		bem_free(bem);
		return r;
	}
	r = totais_add(dec, "BENSANTERIOR", bem->valor_anterior);
	r += totais_add(dec, "BENS", bem->valor);
	if (r) {
		bem_free(bem);
		return r;
	}
	return 0;
}

void bem_salva(struct declaracao *dec, FILE *f)
{
	int i;
	struct bem *j;
	for (i = 0; j = list_get(dec->bens, i); i++) {
		fprintf(f, "bem %d \"%s\" %lld %lld",
			j->codigo, j->descricao, j->valor_anterior, j->valor);
		//attr_printf(f, bem->attr);
		fprintf(f, "\n");
	}
}

static struct cmd cmd_bem = {
	.name = "bem",
	.run = run_bem,
};

int bem_cmd_init(void)
{
	cmd_add(&cmd_bem);
	return 0;
}
